/*
-- ----------------------------------------------------------------------------
--
--  Object name : hostio.c
--  Revision    : 1.0
--
--  Copyright (c) Inmos Ltd, 1987.
--  All Rights Reserved.
--
--  DESCRIPTION
--    TDS Server  SUN Implementation specific routines
--
--  NOTES
-- System dependencies:
--   rename() is a routine which takes two filenames, and renames the first
--     to the second (standard UNIX system call).
--
--   The other routines called are all from the UNIX curses package.
--
--  HISTORY
--     ibm-pc   03-Apr-88  CREATION.
--     SUN      15-Jun-89.
-- ----------------------------------------------------------------------------
*/

/*#include "inmos.h"*/
/*#include "srvconst.h"*/
#include "termp.h"
#include <sgtty.h>

char *tgetstr () ;
char *getenv () ;

int first_screen_line = FIRST_SCREEN_LINE ;
int first_screen_col  = FIRST_SCREEN_COL ;

int last_screen_line, last_screen_col ;

/* External variables for 'tputs' */

short ospeed ;
char  PC ;

/* Original state of terminal IO processing */

struct sgttyb original ;

/* Terminal capabilities */

char term_area [2048] ;

char *cm, *cd, *cl, *ce, *al, *dl, *bl, *ks, *ke, *cs, *pc, *sr, *DO ;

int lines, columns ;

/* Building string for screen operation */

char cap_str [1024] ;

char *cap_ptr ;

/*
-- ---------------------------------------------------------------------------
-- int outc - character-processing routine for 'tputs'.
--
-- Input Parameters:
--   (int) ch - The character to be written.
--
-- Output Parameters:
--   None.
-- ---------------------------------------------------------------------------
*/
int outc (ch)
char ch ;
{
  *(cap_ptr++) = ch ;
  return (1) ;
}

/*
-- ---------------------------------------------------------------------------
-- void cap_print - write cap_str to terminal
-- ---------------------------------------------------------------------------
*/
void cap_print ()
{
  int result ;
  result = write (1, cap_str, cap_ptr - cap_str) ;
}

/*
-- ---------------------------------------------------------------------------
-- void Print - direct write of string to screen
--
-- Input Parameters:
--   (char *) str - The string to be written.
--
-- Output Parameters:
--   None.
-- ---------------------------------------------------------------------------
*/
void Print (str)
char *str ;
{
  int result ;
  char *p ;
  for (p = str; *p; p++)
    ;
  result = write (1, str, p - str) ;
}

void put_cap (cap, affect)
char *cap ;
int affect ;
{
  cap_ptr  = cap_str ;
  tputs (cap, affect, outc) ;
  cap_print () ;
}

/*
-- ---------------------------------------------------------------------------
-- void read_termcap - Read "terminal capabilities" from 'term_desc'.
-- ---------------------------------------------------------------------------
*/
void read_termcap (term_desc)
char *term_desc ;
{
  char *entry ;

  entry  = term_area ;

  cm = tgetstr ("cm", &entry) ;
  cd = tgetstr ("cd", &entry) ;
  cl = tgetstr ("cl", &entry) ;
  ce = tgetstr ("ce", &entry) ;
  al = tgetstr ("al", &entry) ;
  dl = tgetstr ("dl", &entry) ;
  bl = tgetstr ("bl", &entry) ;
  ks = tgetstr ("ks", &entry) ;
  ke = tgetstr ("ke", &entry) ;
  cs = tgetstr ("cs", &entry) ;
  pc = tgetstr ("pc", &entry) ;
  sr = tgetstr ("sr", &entry) ;
  DO = tgetstr ("do", &entry) ;

  lines   = tgetnum ("li") ;
  columns = tgetnum ("co") ;
}

/*
-- ---------------------------------------------------------------------------
-- void term_init - Terminal initialisation.
-- ---------------------------------------------------------------------------
*/
void term_init ()
{
  /* Go into "keypad transmit" mode, if relevant... */

  if (ks)
    put_cap (ks, 1) ;
}

/*
-- ---------------------------------------------------------------------------
-- void term_close () - Restore terminal to original state.
-- ---------------------------------------------------------------------------
*/
void term_close ()
{
  /* End "keypad transmit" mode, if relevant... */

  if (ke)
    put_cap (ke, 1) ;
}

/*
-- ----------------------------------------------------------------------------
-- void io_processing_init () - 
--
--      Initialise IO processing - switch off echoing, and go into
--      "cbreak" mode (very little line-processing - but do let signals
--      get through).
--      original state of 'sgttyb' structure is saved.
-- ----------------------------------------------------------------------------
*/
void io_processing_init ()
{
  struct sgttyb new ;

  ioctl (0, TIOCGETP, &original) ;

  ioctl (0, TIOCGETP, &new) ;
  new.sg_flags = (new.sg_flags & (~ECHO)) | CBREAK ;
  ioctl (0, TIOCSETP, &new) ;
}

/*
-- ---------------------------------------------------------------------------
-- void io_processing_close - Return IO processing to original state.
-- ---------------------------------------------------------------------------
*/
void io_processing_close ()
{
  ioctl (0, TIOCSETP, &original) ;
}

/*
-- ----------------------------------------------------------------------------
-- void externals_init () - Initialise miscellaneous external variables.
--                          NB: assumes 'lines', 'columns', 'pc' and
--                          'original' already defined.
-- ----------------------------------------------------------------------------
*/
void externals_init ()
{
  /* Used locally and in 'screen.c' */
  last_screen_line = lines   - (first_screen_line + 1) ;
  last_screen_col  = columns - (first_screen_col  + 1) ;

  /* External variable 'ospeed' for 'tputs' */
  ospeed = (short) original.sg_ospeed ;

  /* External variable 'PC' for 'tputs'... */
  if (pc)
    PC = *pc ;
  else
    PC = '\0' ;
}

/*
-- ----------------------------------------------------------------------------
-- void HostioInit - Initialise keyboard/screen handling to state suitable
--                   for TDS editor.
-- ----------------------------------------------------------------------------
*/
int HostioInit ()
{
  int result ;
  char term_desc [1024] ;
  char *terminal ;

  if (terminal = getenv ("TERM"))
    if ((result = tgetent (term_desc, terminal)) > 0)
    {
      read_termcap (term_desc) ; 
      term_init () ;
      io_processing_init () ;
      externals_init () ;
      return (1) ;
    }
    else 
    {
      printf ("ERROR: Unrecognised terminal \"%s\"\n", terminal) ;
      return (0) ;
    }
  else
  {
    printf ("ERROR: No TERM environment variable\n") ;
    return (0) ;
  }
}
/*
-- ---------------------------------------------------------------------------
-- void HostioClose - Return terminal to normal state.
-- ---------------------------------------------------------------------------
*/
void HostioClose ()
{
  io_processing_close () ;
  term_close () ;
}

/*
-- ---------------------------------------------------------------------------
-- int GetKey - Return a key from the keyboard.
--
-- Input Parameters:
--   None.
--
-- Output Parameters:
--   (int) Return Key read from Keyboard.
-- ---------------------------------------------------------------------------
*/
int GetKey ()
{
  return (getchar ());
}
/*
-- ---------------------------------------------------------------------------
-- void PutChar - Writes a char to the screen.
--
-- Input Parameters:
--   (int) ch - The character to be written.
--
-- Output Parameters:
--   None.
-- ---------------------------------------------------------------------------
*/
void PutChar (ch)
int ch ;
{
  char data = (char) ch ;
  int result ;
  result = write (1, &data, 1) ;
  /* using following led to problems with buffer not getting flushed.
  putchar (ch) ;
  */
}

/*
-- ---------------------------------------------------------------------------
-- void GotoXY - Move the cursor
--
-- Input Parameters:
--   (int) x - X coord.
--   (int) y - Y coord.
--
-- Output Parameters:
--   None.
--
-- ---------------------------------------------------------------------------
*/
void GotoXY (X, Y)
int X, Y;
{
  put_cap (tgoto (cm, X, Y), 1) ;
}

/*
-- ---------------------------------------------------------------------------
-- void ClearScreen - Clear the whole screen
--
-- ---------------------------------------------------------------------------
*/
void ClearScreen ()
{
  put_cap (cl, lines) ;
}

/*
-- ---------------------------------------------------------------------------
-- void ClearEol - Clear to end of line
--
-- ---------------------------------------------------------------------------
*/
void ClearEol ()
{
  put_cap (ce, 1) ;
}

/*
-- ---------------------------------------------------------------------------
-- void ClearEos - Clear to end of screen
--
-- ---------------------------------------------------------------------------
*/
void ClearEos ()
{
  put_cap (cd, lines) ;
}

/*
-- ---------------------------------------------------------------------------
-- void RingBell - Ring the bell.
--
-- ---------------------------------------------------------------------------
*/
void RingBell ()
{
  put_cap (bl, 1) ;
}

/*
-- ---------------------------------------------------------------------------
-- int InsLine - Insert blank line at current cursor position
--
-- Input Parameters:
--   int X, Y: current cursor position
-- Output 
--   int     : 0 => operation failed (terminal too dumb)
--
-- ---------------------------------------------------------------------------
*/
int InsLine (X, Y)
int X, Y ;
{
  if (al)          /* Direct add-line cabability */
  {
    put_cap (al, lines - Y) ;
    return (1) ;
  }
  else
    if (cs)   /* VT100-like */
    {
      if (last_screen_line > Y)
      {
        /*
        1) Set scrolling region from Y to bottom of page.     
        2) Put cursor back where it was.                     
        3) Reverse index.                                  
        4) Reset scrolling region to whole page.         
        */

        cap_ptr  = cap_str ;
        tputs (tgoto (cs, last_screen_line, Y), 1, outc) ;
        tputs (tgoto (cm, X, Y), 1, outc) ;
        tputs (sr, 1, outc) ;
        tputs (tgoto (cs, last_screen_line, first_screen_line), 1, outc) ;
        cap_print () ;
      }
      else
      {
        /*
        1) Put cursor at start of line.                       
        2) Clear to end of line.                             
        3) Put cursor back where it was.                    
        */

        cap_ptr  = cap_str ;
        tputs (tgoto (cm, 0, Y), 1, outc) ;
        tputs (ce, 1, outc) ;
        tputs (tgoto (cm, X, Y), 1, outc) ;
        cap_print () ;
      }
      return (1) ;
    }
    else           /* Terminal too dumb - give up */
      return (0) ;
}

/*
-- ---------------------------------------------------------------------------
-- int DelLine - Delete current line
--
-- Input Parameters:
--   int X, Y: current cursor position
-- Output 
--   int     : 0 => operation failed (terminal too dumb)
--
-- ---------------------------------------------------------------------------
*/
int DelLine (X, Y)
int X, Y ;
{
  if (dl)          /* Direct delete-line cabability */
  {
    put_cap (dl, lines - Y) ;
    return (1) ;
  }
  else
    if (cs)   /* VT100-like */
    {
      if (last_screen_line > Y)
      {
        /*
        1) Set scrolling region from Y to bottom of page.     
        2) Put cursor at bottom margin of scrolling region.   
        3) Cursor down (force a scroll).                     
        4) Put cursor back where it was.                      
        5) Reset scrolling region to whole page.              
        */

        cap_ptr  = cap_str ;
        tputs (tgoto (cs, last_screen_line, Y), 1, outc) ;
        tputs (tgoto (cm, X, last_screen_line), 1, outc) ;
        tputs (DO, 1, outc) ;
        tputs (tgoto (cm, X, Y), 1, outc) ;
        tputs (tgoto (cs, last_screen_line, first_screen_line), 1, outc) ;
        cap_print () ;
      }
      else
      {
        /*
        1) Put cursor at start of line.                       
        2) Clear to end of line.                             
        3) Put cursor back where it was.                    
        */

        cap_ptr  = cap_str ;
        tputs (tgoto (cm, 0, Y), 1, outc) ;
        tputs (ce, 1, outc) ;
        tputs (tgoto (cm, X, Y), 1, outc) ;
        cap_print () ;
      }
      return (1) ;
    }
    else           /* Terminal too dumb - give up */
      return (0) ;
}

/*
-- ---------------------------------------------------------------------------
-- int DoRename  - Rename a file!!!
--
-- Input Parameters:
--   (BYTE *) OldName - Old file Name.
--   (BYTE *) NewName - New file Name.
--
-- Output Parameters:
--   (int) Return result of rename.
-- ---------------------------------------------------------------------------
*/
int DoRename (OldName, NewName)
char *OldName;
char *NewName;
{
  return(rename(NewName, OldName));
}

